#!/usr/bin/env python

"""
Unit tests for the database faker app
"""

from db_faker.lib.utils import get_root_path
from db_faker.commands.validate_schema import ValidateSchema
from db_faker.commands.generate_data import GenerateData
from db_faker.cli import DatabaseFakerApp

app = DatabaseFakerApp()
valid_schema = get_root_path() / 'tests' / 'sample-schema.json'
invalid_schema = get_root_path() / 'tests' / 'sample-invalid-schema.json'


def test_check_json_schema_loads():
    """Checks that the JSON schema is loading"""
    cmd = ValidateSchema(app, [])
    schema = cmd.json_schema
    assert schema is not None


def test_valid_schema():
    """Checks that the command is able to validate valid schemas"""
    cmd = ValidateSchema(app, ["-s", str(valid_schema)])
    parser = cmd.get_parser("validate-schema")
    cmd.take_action(parser.parse_args(cmd.app_args))


def test_generate_data():
    """Checks that data generation is working fine"""
    cmd = GenerateData(app, ["-s", str(valid_schema), "-o", "console"])
    parser = cmd.get_parser("generate-data")
    cmd.take_action(parser.parse_args(cmd.app_args))




import logging
import json
import db_faker.lib.utils as utils

from cliff.command import Command
from db_faker.lib.schema import validate_schema


class ValidateSchema(Command):
    """
    Command that will validate the given JSON file to the JSON schema
    """
    def __init__(self, app, app_args):
        super(ValidateSchema, self).__init__(app, app_args, cmd_name="validate-schema")
        self.log = logging.getLogger(__name__)
        schema_path = utils.get_root_path() / 'db_faker' / 'schemas' / 'dbSchema.json'
        with open(schema_path) as path:
            self.json_schema = json.load(path)

    def get_parser(self, prog_name):
        parser = super(ValidateSchema, self).get_parser(prog_name)
        parser.add_argument('-s', '--schema', default='./schema.json', metavar='',
                            help='The schema to be used for generating the fake data', type=str)
        return parser

    def take_action(self, parsed_args):
        self.log.debug("Selected schema: %s", parsed_args.schema)

        schema = parsed_args.schema
        schema_file = utils.get_file_abs_path(schema)

        if schema_file.exists():
            vr = validate_schema(schema_file, self.json_schema)
            if vr.valid:
                self.log.info("The schema is valid")
            else:
                self.log.error("The schema passed is not valid")
                self.log.error(vr.validation_errors)
        else:
            self.log.error("The file could not be found")



#!/usr/bin/env python

"""Tests for the utils module."""

import pytest

from db_faker.lib import utils


# Get the absolute path of the current directory
current_absolute_path = utils.get_root_path() / 'tests'


@pytest.mark.parametrize("path, expected", [
    (str(current_absolute_path / 'sample-schema.json'), True),
    ("./sample-schema.json", True),
    (str(current_absolute_path / 'invalid-file.txt'), False),
    ("./invalid-file.txt", False)])
def test_is_file(path: str, expected):
    """
    Tests files exists in unix platforms
    """
    file_exists = utils.file_exists(path)

    assert file_exists == expected



"""
Utility library for managing the JSON schemas
"""
import json
import db_faker.lib.utils as utils

from jsonschema import validate, ValidationError
from pathlib import Path


class SchemaValidationResult:
    """
    Class that stores the result of a JSON schema validation
    """
    def __init__(self, valid: bool, error_message: str = None):
        self.valid = valid
        self.validation_errors = error_message


def validate_schema(path: Path, json_schema) -> SchemaValidationResult:
    with open(utils.get_file_abs_path(path)) as schema_file:
        schema_to_validate_json = json.load(schema_file)
        try:
            # Validate the desired JSON file against the JSON schema for the database definition
            validate(schema_to_validate_json, json_schema)
            return SchemaValidationResult(True)
        except ValidationError as e:
            return SchemaValidationResult(False, error_message=e.message)



import logging
import abc
import six

from pathlib import Path


@six.add_metaclass(abc.ABCMeta)
class OutputHandler:
    """
    Base class for handling the output of the CLI application.
    """

    @abc.abstractmethod
    def handle(self, output: str):
        """
        Method to be overridden by subclasses to perform the action on the output
        """


class ConsoleHandler(OutputHandler):
    """
    OutputHandler that logs the output to the console via loggers
    """

    def __init__(self, logger_name: str):
        self.logger = logging.getLogger(logger_name)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        # Loggers do not need to be closed
        pass

    def handle(self, output: str):
        self.logger.info(output)


class FileHandler(OutputHandler):
    """
    OutputHandler that logs the output to a specified file
    """

    def __init__(self, file: str):
        self.file = open(Path(file), "w")

    def __enter__(self):
        return self

    def __exit__(self, type, val, traceback):
        self.file.close()

    def handle(self, output: str):
        self.file.write(output + "\n")



import logging
import json
import db_faker.lib.utils as utils

from pathlib import Path
from itertools import repeat
from cliff.command import Command
from db_faker.lib.schema import validate_schema
from db_faker.lib.output_handlers import ConsoleHandler, FileHandler
from faker import Faker


def output_param_is_valid(output: str, force: bool) -> bool:
    """
    This method will check if the output parameter is either "console", or a valid file path that does not exists.
    """
    if "console" == output:
        return True
    else:
        return True if force else utils.file_not_exists(output)


class GenerateData(Command):
    """
    Command that will generate the fake data from the schema
    """

    def __init__(self, app, app_args):
        super(GenerateData, self).__init__(app, app_args, cmd_name="generate-data")
        self.log = logging.getLogger(__name__)
        schema_path = utils.get_root_path() / 'db_faker' / 'schemas' / 'dbSchema.json'
        with open(schema_path) as path:
            self.json_schema = json.load(path)

    def get_parser(self, prog_name):
        parser = super(GenerateData, self).get_parser(prog_name)
        parser.add_argument('-s', '--schema', default='./schema.json', metavar='',
                            help='The schema to be used for generating the fake data', type=str)
        parser.add_argument('-o', '--output', default='console', metavar='',
                            help='The output of the command. It can be either console, '
                                 'or a path to a file.',
                            type=str)
        parser.add_argument('-f', '--force', help='Forces the overwrite of the destination file', default=False,
                            type=bool)
        return parser

    def take_action(self, parsed_args):
        self.log.debug("Selected schema: %s", parsed_args.schema)
        self.log.debug("Selected output: %s", parsed_args.output)

        schema = parsed_args.schema
        schema_file = utils.get_file_abs_path(schema)

        if schema_file.exists():
            vr = validate_schema(schema_file, self.json_schema)
            if vr.valid:
                self.log.debug("The schema is valid")
                if output_param_is_valid(parsed_args.output, parsed_args.force):
                    # Actually create the data!
                    self.generate_data_from_schema(schema_file, parsed_args.output)
                else:
                    self.log.error("The output parameter is not valid, please specify either 'console' "
                                   "or a valid pathname")
            else:
                self.log.error("The schema passed is not valid")
                self.log.error(vr.validation_errors)
        else:
            self.log.error("The file could not be found")

    def generate_data_from_schema(self, schema: Path, output: str):
        with (generate_handler(output, self.log.name)) as output_handler:

            with open(schema) as schema_file:
                schema_json = json.load(schema_file)

                # Generate the Faker instance
                fake = Faker(schema_json["locale"]) if schema_json.__contains__("locale") else Faker()

                db_name = schema_json["name"]

                # We need to create here, for each table, a set of rows like the following:
                # INSERT INTO <table_name> ([<fields>]) values ([<values>])
                tables = schema_json["tables"]
                for table in tables:
                    n_rows = table["rows"] if table.__contains__("rows") else 50
                    for _ in repeat(None, n_rows):
                        insert_stmt = generate_row_data(fake, db_name, table["name"], table["fields"])
                        output_handler.handle(insert_stmt)


def generate_handler(output: str, logger: str):
    if "console" == output:
        return ConsoleHandler(logger_name=logger)
    else:
        return FileHandler(file=output)


def generate_row_data(fake: Faker, db_name: str, table_name: str, fields: list) -> str:
    sql_insert = f"INSERT INTO {db_name}.{table_name} ("
    sql_values = " VALUES ("

    for field in fields:
        sql_insert += field["name"] + ","
        value = generate_fake_data(fake, field["type"])
        sql_values += f"\"{value}\","

    # Remove last comma and end statement
    sql_insert = sql_insert[:-1] + ")"
    sql_values = sql_values[:-1] + ");"

    return sql_insert + sql_values


def generate_fake_data(fake: Faker, value_type: str):
    switch = {
        "name": fake.name(),
        "username": fake.user_name(),
        "address": fake.address(),
        "creditcard": fake.credit_card_number(),
        "company": fake.company(),
        "phone": fake.phone_number(),
        "email": fake.ascii_safe_email(),
        "vehicle_license": fake.license_plate(),
        "iban": fake.iban(),
        "color": fake.color_name(),
        "string": fake.text(),
        "integer": str(fake.random_int(max=99999)),
        "numeric": str(fake.pyfloat()),
        "date": fake.date(),
        "time": fake.time(),
        "timestamp": fake.iso8601(),
        "binary": str(fake.binary(length=64)),
        "bool": str(fake.boolean()),
        "uuid": fake.uuid4(),
        "inet": fake.ipv4(),
        "macaddr": fake.mac_address()
    }

    return switch.get(value_type)



import os
import db_faker

from pathlib import Path


def get_root_path() -> Path:
    """
    Returns the root path of the module
    """
    return Path(db_faker.__file__).parent.parent


def file_exists(path: str) -> bool:
    """
    Returns a boolean indicating if the file referenced by a string pathname do exist
    """
    if os.path.isabs(path):
        abs_path = Path(path)
    else:
        abs_path = Path.cwd().joinpath(path).resolve()

    return abs_path.is_file()


def file_not_exists(path: str) -> bool:
    """
    Returns a boolean indicating if the file referenced by a string pathname do not exist.
    """
    if os.path.isabs(path):
        abs_path = Path(path)
    else:
        abs_path = Path.cwd().joinpath(path).resolve()

    return not abs_path.exists()


def get_file_abs_path(path: str) -> Path:
    """
    Returns the absolute path of the provided string path
    """
    return Path(path) if os.path.isabs(path) else Path.cwd().joinpath(path).resolve()



"""Console script for db_faker."""
import sys

from cliff.app import App
from cliff.commandmanager import CommandManager


class DatabaseFakerApp(App):

    def __init__(self):
        super(DatabaseFakerApp, self).__init__(
            description='Database Faker CLI app',
            version='0.0.1',
            command_manager=CommandManager('db_faker.commands'),
            deferred_help=True
        )

    def initialize_app(self, argv):
        self.LOG.debug("Initializing Database Faker CLI app")

    def prepare_to_run_command(self, cmd):
        self.LOG.debug("Preparing to run command %s", cmd.__class__.__name__)

    def clean_up(self, cmd, result, err):
        self.LOG.debug("Cleaning up %s", cmd.__class__.__name__)
        if err:
            self.LOG.debug("There was an error: %s", err)


def main(argv=sys.argv[1:]):
    """Console script for db_faker."""
    cli = DatabaseFakerApp()
    return cli.run(argv)


if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))



#!/usr/bin/env python

"""The setup script."""
import os
import sys

from setuptools import setup, find_packages
from setuptools.command.install import install

VERSION = '0.1.1'

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

# Generating the requirements of the package from the requirements.txt file
lib_folder = os.path.dirname(os.path.realpath(__file__))
requirements_file = lib_folder + '/requirements.txt'
requirements = []
if os.path.isfile(requirements_file):
    with open(requirements_file, "r") as reqs:
        requirements = reqs.read().splitlines()

# Requirements for setup the package
setup_requirements = ['pytest-runner', ]

# Requirements for testing the package
test_requirements = ['pytest>=6', ]


# https://circleci.com/blog/continuously-deploying-python-packages-to-pypi-with-circleci/
class VerifyVersionCommand(install):
    """Custom command to verify that the git tag matches our version"""
    description = 'verify that the git tag matches our version'

    def run(self):
        tag = os.getenv('CIRCLE_TAG')

        if tag != VERSION:
            info = "Git tag: {0} does not match the version of this app: {1}".format(
                tag, VERSION
            )
            sys.exit(info)


# Setup function
setup(
    author="Pablo Barrientos",
    python_requires='>=3.5',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="CLI application that handles the creation of fake data to databases",
    entry_points={
        'console_scripts': [
            'db_faker=db_faker.cli:main',
        ],
        "db_faker.commands": [
            "validate-schema=db_faker.commands.validate_schema:ValidateSchema",
            "generate-data=db_faker.commands.generate_data:GenerateData"
        ]
    },
    install_requires=requirements,
    license="MIT license",
    long_description=readme + '\n\n' + history,
    long_description_content_type="text/x-rst",
    include_package_data=True,
    keywords='db_faker test databases data',
    name='db_faker',
    packages=find_packages(include=['db_faker', 'db_faker.*']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/PBarri/db-faker',
    version=VERSION,
    zip_safe=False,
    cmd_class={
        'verify': VerifyVersionCommand
    }
)
